"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildExpressionAttributeNames = exports.buildExpressionAttributeValues = exports.buildDynamoQueryExpression = exports.isKvsLogicalOperator = exports.isKvsQueryCondition = void 0;
const crypto_1 = __importDefault(require("crypto"));
const quidproquo_core_1 = require("quidproquo-core");
// TODO: Come up with a better way of generating item name / values
// probably a map ~ however this will most likely never fail.
const getHash = (name) => {
    const c = crypto_1.default.createHash('md5').update(name).digest('hex');
    return c;
};
const getItemName = (name) => {
    return `#${getHash(name)}`;
};
const getValueName = (value) => {
    return `:${getHash(`${typeof value}-${JSON.stringify(value)}`)}`;
};
const buildDynamoQueryExpressionBetween = (query) => {
    if (!query.key || !query.valueB || !query.valueB) {
        throw new Error(`Invalid query condition ${quidproquo_core_1.KvsQueryOperationType.Between}`);
    }
    return `${getItemName(query.key)} BETWEEN ${getValueName(query.valueA)} AND ${getValueName(query.valueB)}`;
};
const buildDynamoQueryExpressionEqual = (query) => {
    return `${getItemName(query.key)} = ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionNotEqual = (query) => {
    return `${getItemName(query.key)} <> ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionLessThan = (query) => {
    return `${getItemName(query.key)} < ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionLessThanOrEqual = (query) => {
    return `${getItemName(query.key)} <= ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionGreaterThan = (query) => {
    return `${getItemName(query.key)} > ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionGreaterThanOrEqual = (query) => {
    return `${getItemName(query.key)} >= ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionIn = (query) => {
    return `${getItemName(query.key)} IN (${query.valueA
        .map((v) => getValueName(v))
        .join(', ')})`;
};
const buildDynamoQueryExpressionExists = (query) => {
    return `attribute_exists(${getItemName(query.key)})`;
};
const buildDynamoQueryExpressionNotExists = (query) => {
    return `attribute_not_exists(${getItemName(query.key)})`;
};
const buildDynamoQueryExpressionBeginsWith = (query) => {
    return `begins_with(${getItemName(query.key)}, ${getValueName(query.valueA)})`;
};
const buildDynamoQueryExpressionContains = (query) => {
    return `contains(${getItemName(query.key)}, ${getValueName(query.valueA)})`;
};
const buildDynamoQueryExpressionNotContains = (query) => {
    return `NOT contains(${getItemName(query.key)}, ${getValueName(query.valueA)})`;
};
const buildDynamoQueryExpressionOr = (query) => {
    return query.conditions.map((c) => `(${buildDynamoQueryExpressionRoot(c)})`).join(' OR ');
};
const buildDynamoQueryExpressionAnd = (query) => {
    return query.conditions.map((c) => `(${buildDynamoQueryExpressionRoot(c)})`).join(' AND ');
};
const isKvsQueryCondition = (query) => {
    return 'key' in query && 'operation' in query;
};
exports.isKvsQueryCondition = isKvsQueryCondition;
const isKvsLogicalOperator = (query) => {
    return 'conditions' in query && 'operation' in query;
};
exports.isKvsLogicalOperator = isKvsLogicalOperator;
const buildDynamoQueryExpressionRoot = (query) => {
    if ((0, exports.isKvsQueryCondition)(query)) {
        switch (query.operation) {
            case quidproquo_core_1.KvsQueryOperationType.Equal:
                return buildDynamoQueryExpressionEqual(query);
            case quidproquo_core_1.KvsQueryOperationType.NotEqual:
                return buildDynamoQueryExpressionNotEqual(query);
            case quidproquo_core_1.KvsQueryOperationType.LessThan:
                return buildDynamoQueryExpressionLessThan(query);
            case quidproquo_core_1.KvsQueryOperationType.LessThanOrEqual:
                return buildDynamoQueryExpressionLessThanOrEqual(query);
            case quidproquo_core_1.KvsQueryOperationType.GreaterThan:
                return buildDynamoQueryExpressionGreaterThan(query);
            case quidproquo_core_1.KvsQueryOperationType.GreaterThanOrEqual:
                return buildDynamoQueryExpressionGreaterThanOrEqual(query);
            case quidproquo_core_1.KvsQueryOperationType.Between:
                return buildDynamoQueryExpressionBetween(query);
            case quidproquo_core_1.KvsQueryOperationType.In:
                return buildDynamoQueryExpressionIn(query);
            case quidproquo_core_1.KvsQueryOperationType.Exists:
                return buildDynamoQueryExpressionExists(query);
            case quidproquo_core_1.KvsQueryOperationType.NotExists:
                return buildDynamoQueryExpressionNotExists(query);
            case quidproquo_core_1.KvsQueryOperationType.BeginsWith:
                return buildDynamoQueryExpressionBeginsWith(query);
            case quidproquo_core_1.KvsQueryOperationType.Contains:
                return buildDynamoQueryExpressionContains(query);
            case quidproquo_core_1.KvsQueryOperationType.NotContains:
                return buildDynamoQueryExpressionNotContains(query);
        }
    }
    else if ((0, exports.isKvsLogicalOperator)(query)) {
        switch (query.operation) {
            case quidproquo_core_1.KvsLogicalOperatorType.And:
                return buildDynamoQueryExpressionAnd(query);
            case quidproquo_core_1.KvsLogicalOperatorType.Or:
                return buildDynamoQueryExpressionOr(query);
        }
    }
    throw new Error(`Invalid query operation: ${JSON.stringify(query)}`);
};
const buildAttributeValue = (value) => {
    switch (typeof value) {
        case 'string':
            return { S: value };
        case 'number':
            return { N: value.toString() };
        case 'boolean':
            return { BOOL: value };
        case 'object':
            if (Array.isArray(value)) {
                // check if all elements in the array are strings
                if (value.every((item) => typeof item === 'string')) {
                    return { L: value.map((item) => ({ S: item })) };
                }
                throw new Error(`Invalid attribute value array content: ${JSON.stringify(value)}`);
            }
            throw new Error(`Invalid attribute value type: ${typeof value}`);
        default:
            throw new Error(`Invalid attribute value type: ${typeof value}`);
    }
};
const buildDynamoQueryExpression = (query) => {
    if (!query) {
        return undefined;
    }
    return buildDynamoQueryExpressionRoot(query);
};
exports.buildDynamoQueryExpression = buildDynamoQueryExpression;
const buildExpressionAttributeValues = (queries) => {
    const values = {};
    const traverse = (query) => {
        if ((0, exports.isKvsQueryCondition)(query)) {
            if (query.valueA !== undefined) {
                const valueNameA = getValueName(query.valueA);
                values[valueNameA] = buildAttributeValue(query.valueA);
            }
            if (query.valueB !== undefined) {
                const valueNameB = getValueName(query.valueB);
                values[valueNameB] = buildAttributeValue(query.valueB);
            }
        }
        else if ((0, exports.isKvsLogicalOperator)(query)) {
            for (const condition of query.conditions) {
                traverse(condition);
            }
        }
    };
    queries.filter((q) => !!q).forEach((q) => traverse(q));
    return values;
};
exports.buildExpressionAttributeValues = buildExpressionAttributeValues;
const buildExpressionAttributeNames = (queries) => {
    const names = {};
    const traverse = (query) => {
        if ((0, exports.isKvsQueryCondition)(query)) {
            const itemName = getItemName(query.key);
            names[itemName] = query.key;
        }
        else if ((0, exports.isKvsLogicalOperator)(query)) {
            for (const condition of query.conditions) {
                traverse(condition);
            }
        }
    };
    queries.filter((q) => !!q).forEach((q) => traverse(q));
    return names;
};
exports.buildExpressionAttributeNames = buildExpressionAttributeNames;
